using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using HPS;

namespace hps_cs_samples
{
    public partial class Sample
    {
        public void Exchange_Publish(WindowKey wk, Canvas canvas, ref View view, Model model, SegmentKey modelKey)
        {
            try
            {
            /* Sample: Exchange_Publish
	         * This sample will show you how to:
	         * - Load an Exchange file and check for IO exceptions
	         * - Output a 3D PDF file through Publish and check for IO exceptions
	         * 
	         * Read more about the topics covered in this sample in our Programming Guide 
	         * - Integration with HOOPS Exchange
	         * - Integration with HOOPS Publish */

            /* This next block of code shows you how to load an Exchange file
	         * 1. Create an ImportOptionsKit and set the segment in which you want the file to be loaded
	         * 2. Use the Exchange.File.Import method to load the desired file, with the options you specified
	         * 3. Since this operation happens on a separate thread, wait for it to complete before proceeding 
	         * 4. Use a try-catch block to capture any Stream IO exceptions 
	         * 5. Obtain an Exchange.CADModel object through the Exchange::ImportNotifier
	         * 6. Obtain the new View from the CADModel object. Attach the new view to the Canvas and delete the old View
	         *		You will need to re-insert geometry that was in the old View, such as lights.
	         *		You will also need to re-apply any operators which were active on the old view.*/
            Exchange.ImportNotifier exchangeNotifier = new Exchange.ImportNotifier();
            try
            {
                string file_in = INPUT_FOLDER + "/Crank-Instructions.prc";
                exchangeNotifier = Exchange.File.Import(file_in, new Exchange.ImportOptionsKit());
                exchangeNotifier.Wait();

                CADModel modelFile = exchangeNotifier.GetCADModel();
                view.Delete();
                view = modelFile.ActivateDefaultCapture();
                canvas.AttachViewAsLayout(view);

                modelKey.InsertDistantLight(new Vector(1, 1, 0));
                view.FitWorld();
            }

            catch (IOException)
            {
                string error_message;
                error_message = "Exchange Import Unsuccessful";
                modelKey.InsertText(new Point(0, 0, 0), error_message);
                view.FitWorld();
                view.Update();
            }

            /* This next block of code shows you how to output a 3D PDF file
	         * 1. Create SprocketPath object, by passing the keys to the canvas, layout, view and model to it
	         * 2. Use the Publish::File::Export method to output a 3D PDF file */
            if (exchangeNotifier.Status() == IOResult.Success)
            {
                bool publishExportSuccess = true;

                try
                {
                    string file_out = OUTPUT_FOLDER + "/exchange_publish_sample.pdf";
                    SprocketPath sprkPath = new SprocketPath(canvas, canvas.GetAttachedLayout(),
                        canvas.GetAttachedLayout().GetAttachedView(),
                        canvas.GetAttachedLayout().GetAttachedView().GetAttachedModel());
                    Publish.File.Export(sprkPath, file_out, new Publish.ExportOptionsKit());
                }

                catch (IOException)
                {
                    string error_message;
                    publishExportSuccess = false;
                    error_message = "Publish Export Unsuccessful";
                    modelKey.InsertText(new Point(0, 0, 0), error_message);
                    view.FitWorld();
                    view.Update();
                }

                if (publishExportSuccess)
                {
                    string success_message = "Import and Export completed successfully\n" +
                        "You can find the 3D PDF here:\n" +
                        OUTPUT_FOLDER + "/exchange_publish_sample.pdf";

                    view.GetAttachedModel().GetSegmentKey().Flush(Search.Type.Everything, Search.Space.SubsegmentsAndIncludes);
                    view.GetAttachedModel().GetSegmentKey().InsertText(new Point(0, 0, 0), success_message);
                    view.FitWorld();
                    view.Update();
                }

            }
            }
            catch (IOException e)
            {
                System.Console.WriteLine(e.Message);
            }
        }
    }
}
