using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using HPS;

namespace hps_cs_samples
{
    public partial class Sample
    {
        public void Exchange_PMI_Highlighting(WindowKey wk, Canvas canvas, ref View view, Model model, SegmentKey modelKey)
        {
            try
            {
            /* Sample: Exchange_PMI_Highlighting
	         * This sample will show you how to:
	         * - Load an Exchange file and check for IO exceptions
	         * - Walk down a component structure
	         * - Verify the type of a component
	         * - Defining named styles
	         * - Highlighting PMIs and geometry referenced by a particular PMI component
	         * - Hide components using highlighting
	         * - Using a KeyPath to operate on a particular instance of geometry
	         * 
	         * Read more about the topics covered in this sample in our Programming Guide 
	         * - Integration with HOOPS Exchange
	         * - Section 6.2:		Highlighting 
	         * - Section 1.3.1:		Using Keys and Objects*/

            /* This next block of code shows you how to load an Exchange file
             * 1. Create an ImportOptionsKit
             *		NOTE: It is required to import BRep data to access geometry referenced by PMI
             * 2. Use the Exchange::File::Import method to load the desired file, with the options you specified
             * 3. Since this operation happens on a separate thread, wait for it to complete before proceeding 
             * 4. Use a try-catch block to capture any Exchange IO exceptions 
             * 5. Obtain a CADModel object through the Exchange::ImportNotifier */
            CADModel cadModel = new CADModel();
	        Exchange.ImportNotifier exchangeNotifier;
	        try {
		        string file_in = INPUT_FOLDER + "/PartWithPMI.CATPart";
		        Exchange.ImportOptionsKit importOptionsKit = new Exchange.ImportOptionsKit();
		        importOptionsKit.SetBRepData(true);
		        exchangeNotifier = Exchange.File.Import(file_in, importOptionsKit);
		        exchangeNotifier.Wait();

		        cadModel = exchangeNotifier.GetCADModel();
	        }

	        catch(IOException) {
		        string error_message;
		        error_message = "Exchange Import Unsuccessful";
		        modelKey.InsertText(new Point(0, 0, 0), error_message);
		        view.FitWorld();
		        view.Update();
	        }

            canvas.GetWindowKey().GetCullingControl().SetExtent(0).SetDeferralExtent(0);

	        /* This block of code creates the styles used to highlight components later on in this sample:
	         * 1. In order for a named style to be used it has to be set on a Portfolio which is visible by the 
	         *		segment attempting to use the named style
	         * 2. Obtain a PortfolioKey from the view segment, by using a PorfolioControl object and calling its
	         *		ShowTop() method
	         * 3. In order to define a named style, we need a name for the style we want to define, and a segment
	         *		which contains the attributes for the style.
	         * 4. yellowSegment is a segment which contains attribute to color faces, lines and text yellow. This
	         *		segment defined the yellowHighlight named style, which is associated with the portfolio key obtained
	         *		in step 2.
	         * 5. The other two named styles defined in this sample are redHighlight which colors faces, lines and text
	         *		red, and hidePMI, which turns off visibility for faces, lines and text */
	        PortfolioKey pKey;
	        SegmentKey yellowSegment = view.GetSegmentKey().Subsegment();
	        yellowSegment.GetMaterialMappingControl().SetFaceColor(new RGBAColor(1, 1, 0, 1))
		        .SetTextColor(new RGBAColor(1, 1, 0, 1))
		        .SetLineColor(new RGBAColor(1, 1, 0, 1));
	        view.GetSegmentKey().GetPortfolioControl().ShowTop(out pKey);
	        pKey.DefineNamedStyle("yellowHighlight", yellowSegment);

	        SegmentKey redSegment = view.GetSegmentKey().Subsegment();
	        redSegment.GetMaterialMappingControl().SetFaceColor(new RGBAColor(1, 0, 0, 1))
		        .SetTextColor(new RGBAColor(1, 0.65f, 0, 1))
		        .SetLineColor(new RGBAColor(1, 0.65f, 0, 1));
	        pKey.DefineNamedStyle("redHighlight", redSegment);

	        SegmentKey hideSegment = view.GetSegmentKey().Subsegment();
	        hideSegment.GetVisibilityControl().SetFaces(false)
		        .SetText(false)
		        .SetLines(false);
	        pKey.DefineNamedStyle("hidePMI", hideSegment);

	        /* This block of code sets up our scene:
	         * We will have two subwindows, each including an instance of the CAD Model we have imported:
	         * 1. Activate the default capture from the CAD Model
	         * 2. Include the default capture into two segments, these two segments will house the two subwindows
	         * 3. Change the background color of the window to make the PMI better visible */
	        SegmentKey view1, view2;
	        view.GetSegmentKey().GetMaterialMappingControl().SetWindowColor(new RGBAColor(0.29f, 0, 0.51f, 1));
	        view1 = view.GetSegmentKey().Subsegment();
	        view2 = view.GetSegmentKey().Subsegment();
	        View defaultView = cadModel.ActivateDefaultCapture();
	        IncludeKey view1Include = view1.IncludeSegment(defaultView.GetSegmentKey());
	        IncludeKey view2Include = view2.IncludeSegment(defaultView.GetSegmentKey());

	        view1.GetSubwindowControl().SetSubwindow(new HPS.Rectangle(-1, 0, -1, 1)).SetBorder(Subwindow.Border.Inset);
	        view1.InsertDistantLight(new Vector(1, 1, 0));
	        view2.GetSubwindowControl().SetSubwindow(new HPS.Rectangle(0, 1, -1, 1)).SetBorder(Subwindow.Border.Inset);
	        view2.InsertDistantLight(new Vector(1, 1, 0));
	
	        view.FitWorld();
	        view.GetSegmentKey().GetModellingMatrixControl().Rotate(0, 0, 90);
	        view.Update();

	        /* This code shows how to walk the component structure of the CAD Model
	         * For this particular model we assume that we know where the PMI data we want to highlight
	         * resides in the component structure.
	         * At any point, when using a component, it is possible to look at what type of
	         * component it represents and performs different actions accordingly. */
	        Component[] rootComponent = cadModel.GetSubcomponents();
	        Component[] parts = rootComponent[0].GetSubcomponents();
	        Component[] partsSubcomponents = parts[0].GetSubcomponents();

	        Component pmi1, pmi2, pmi3, pmi4;
	        pmi1 = partsSubcomponents[18];
	        pmi2 = partsSubcomponents[19];
	        pmi3 = partsSubcomponents[20];
	        pmi4 = partsSubcomponents[26];

	        if (pmi1.GetComponentType() != Component.ComponentType.ExchangePMIGDT)
	        {
		        //This code snippet is here just to show how to discriminate between 
		        //component types at runtime
		        
                //Components have metadata associated with them.
                //It is possible to check the name of a component using this method
                StringMetadata pmIName = new StringMetadata(pmi1.GetMetadata("Name"));
	        }

            /* This code shows how to obtain KeyPath objects for the PMI components we are going to select
             * Key paths are needed to obtain the specific instance of the geometry we want to highlight 
             * The key path obtained from the Component::GetKeyPath method stops at the segment where the
             * default capture of the CAD model is included. The extra parts of the key paths, which will
             * go all the way to the window key, will be added later
             * 
             * Each component can have multiple key paths depending on whether a component is included in
             * multiple parts of the model of whether the model is included in multiple views, layouts or
             * canvases. 
             * In this case we are only interested in the first key path */
            List<KeyPath> keyPaths = new List<KeyPath>();
	        KeyPath[] pmi1KeyPath, pmi2KeyPath, pmi3KeyPath, pmi4KeyPath;
	        pmi1KeyPath = Component.GetKeyPath(pmi1);
	        pmi2KeyPath = Component.GetKeyPath(pmi2);
	        pmi3KeyPath = Component.GetKeyPath(pmi3);
	        pmi4KeyPath = Component.GetKeyPath(pmi4);
            keyPaths.Add(pmi1KeyPath[0]);
            keyPaths.Add(pmi2KeyPath[0]);
            keyPaths.Add(pmi3KeyPath[0]);
            keyPaths.Add(pmi4KeyPath[0]);

	        /* This block of code walks the KeyPathArray obtained above, and adds to its KeyPath objects
	         * the necessary keys to fully resolve the path all the way to the window.
	         * 
	         * In order to highlight the PMI component we do the following:
	         * 1. Obtain a HighlightOptionsKit and set a named style on it.
	         * 2. From the window key, obtain a HighlightControl object and call its Highlight method
	         * 3. Passing a fully formed keypath to the highlight method ensures that only the instance of
	         *		a geometry contained in the key path will be highlighted*/
            HighlightOptionsKit highlightOptions = new HighlightOptionsKit();
	        foreach (KeyPath kp in keyPaths)
	        {
		        Key[] keys;
		        kp.ShowKeys(out keys);
                List<Key> keyList = new List<Key>(keys);
                keyList.Add(view1Include);
                keyList.Add(canvas.GetAttachedLayout().GetAttachedViewIncludeLink(0));
                keyList.Add(canvas.GetAttachedLayoutIncludeLink());
                keyList.Add(wk);

                KeyPath highlightKeyPath = new KeyPath(keyList.ToArray());
		        highlightOptions.SetStyleName("yellowHighlight");
		        wk.GetHighlightControl().Highlight(highlightKeyPath, highlightOptions);
	        }

	        /* This block of code repeats what is done above, but in this case, the key path is made
	         * to reference the right subwindow, and the highlight style is changed to hidePMI, which 
	         * will make the PMI highlighted above, invisible in the right subwindow */
	        highlightOptions.SetStyleName("hidePMI");
	        foreach (KeyPath kp in keyPaths)
	        {
		        Key[] keys;
		        kp.ShowKeys(out keys);
                List<Key> keyList = new List<Key>(keys);
                keyList.Add(view2Include);
                keyList.Add(canvas.GetAttachedLayout().GetAttachedViewIncludeLink(0));
                keyList.Add(canvas.GetAttachedLayoutIncludeLink());
                keyList.Add(wk);

                KeyPath hiddenKeyPath = new KeyPath(keyList.ToArray());
		        wk.GetHighlightControl().Highlight(hiddenKeyPath, highlightOptions);
	        }

	        /* This block of code shows how to highlight a piece of geometry referenced by PMI
	         * 1. Obtain the PMI component for which you want to obtain its associated geometry
	         * 2. Make sure that the model was imported with BRep Data
	         * 3. Obtain the PMI subcomponents
	         * 4. Obtain the key path for the subcomponents and fully resolve it all the way to the
	         *		window key
	         * 5. Set the desired highlighting style and pass the key path to the Highlight method*/
	        Component[] associatedGeometry = pmi1.GetSubcomponents();
	        keyPaths.Clear();
	        foreach (Component reference in associatedGeometry)
	        {
		        KeyPath[] referenceKeyPath = Component.GetKeyPath(reference);
		        keyPaths.Add(referenceKeyPath[0]);
	        }

	        highlightOptions.SetStyleName("redHighlight");
	        foreach (KeyPath kp in keyPaths)
	        {
		        Key[] keys;
		        kp.ShowKeys(out keys);
                List<Key> keyList = new List<Key>(keys);
                keyList.Add(view1Include);
                keyList.Add(canvas.GetAttachedLayout().GetAttachedViewIncludeLink(0));
                keyList.Add(canvas.GetAttachedLayoutIncludeLink());
                keyList.Add(wk);

                KeyPath highlightKeyPath = new KeyPath(keyList.ToArray());
		        wk.GetHighlightControl().Highlight(highlightKeyPath, highlightOptions);
	        }
	
	        /* The next block adds text description to both subwindows. For each subwindow the following
	         * operations are performed:
	         * 1. A subsegment is created, and depth range is set on it to make sure that the geometry in this
	         *		segment will always appear on top
	         * 2. Text visibility is enabled through the VisibilityControl
	         * 3. A stretched camera is added to the subsegment
	         * 4. The text attributes are modified as needed using the TextAttributeControl
	         * 5. The color of the text is set to white using the MaterialMappingControl
	         * 6. Text is inserted */
	        SegmentKey view1Description = view1.Subsegment();
	        view1Description.GetDrawingAttributeControl().SetDepthRange(0.0f, 0.1f);
	        view1Description.GetVisibilityControl().SetEverything(false).SetText(true);
	        view1Description.GetCameraControl().SetPosition(new Point(0, 0, -5)).
				SetTarget(new Point(0, 0, 0)).
				SetUpVector(new Vector(0, 1, 0)).
				SetField(2, 2).
				SetProjection(Camera.Projection.Stretched);
	        view1Description.GetTextAttributeControl().SetSize(0.02f, Text.SizeUnits.SubscreenRelative).
				SetAlignment(Text.Alignment.Center, Text.ReferenceFrame.WorldAligned, Text.Justification.CenterJustified);
	        view1Description.GetMaterialMappingControl().SetTextColor(new RGBAColor(1, 1, 1, 1));
	        view1Description.InsertText(new Point(0, -0.85f, 0), "PMI highlighted in yellow");
	        view1Description.InsertText(new Point(0, -0.80f, 0), "referenced geometry highlighted in red");

	        SegmentKey view2Description = view2.Subsegment();
	        view2Description.GetDrawingAttributeControl().SetDepthRange(0.0f, 0.1f);
	        view2Description.GetVisibilityControl().SetEverything(false).SetText(true);
	        view2Description.GetCameraControl().SetPosition(new Point(0, 0, -5)).
		        SetTarget(new Point(0, 0, 0)).
		        SetUpVector(new Vector(0, 1, 0)).
		        SetField(2, 2).
		        SetProjection(Camera.Projection.Stretched);
	        view2Description.GetTextAttributeControl().SetSize(0.02f, Text.SizeUnits.SubscreenRelative).
		        SetAlignment(Text.Alignment.Center, Text.ReferenceFrame.WorldAligned, Text.Justification.CenterJustified);
	        view2Description.GetMaterialMappingControl().SetTextColor(new RGBAColor(1, 1, 1, 1));
	        view2Description.InsertText(new Point(0, -0.85f, 0), "PMI hidden");

            }
            catch (IOException e)
            {
                System.Console.WriteLine(e.Message);
            }

        }
    }
}